<?php

namespace App\controleur;

use App\config\Validation;
use App\gateway\Connection;
use App\gateway\ImageSaver;
use App\gateway\OffreGateway;
use App\modele\UtilisateurModele;

use App\config;

class UtilisateurControleur
{
    private  \App\modele\UtilisateurModele  $userModel;

    public function __construct()
    {
        global $twig;

        $this->userModel = new \App\modele\UtilisateurModele();

        if (!isset($_REQUEST["action"])) {
            //$action = NULL;
            $action = "consultOffers";
        } else {
            $action = \App\config\Validation::nettoyerString($_REQUEST["action"]);
        }
        switch ($action) {
            case NULL:
            case "accueil":
                $this->accueil();
                break;
            case "inscription_success":
                $this->inscription_success();
                break;
            case "connection":
                $this->connection();
                break;
            case "inscription":
                $this->inscription();

                break;
            case "consultOffers":
                $this->consultOffers();
                break;
            case "consulterProfilLimite":
                $this->consulterProfilLimite();
                break;
            case "createOffer":
                $this->createOffer();
                break;
            case "createOfferForm":
                $this->createOfferForm();
                break;
            case "displayOffer":
                $this->displayOffer();
                break;
            default:
                $dVueErreur[] = "Action inconnue ou non autorisée";
                echo $twig->render("erreur.html", ['dVueErreur' => $dVueErreur]);

        }
    }

    protected function connection()
    {
        global $twig;
        $dVueErreur = []; // Tableau pour stocker les erreurs, le cas échéant
        $userModel = new \App\modele\UtilisateurModele();

        if (isset($_POST['email'], $_POST['password'])) {
            $email = \App\config\Validation::nettoyerString($_POST['email']);
            $motDePasse = \App\config\Validation::nettoyerString($_POST['password']);
            $utilisateur = $userModel->connection($email, $motDePasse);
            if ($utilisateur instanceof \App\metier\Alumni) {
                $_SESSION['utilisateur'] = $utilisateur;
                header('Location: index.php?action=accueil');
                exit();
            } else {
                $dVueErreur[] = "L'adresse email ou le mot de passe est incorrect.";
            }
        }
        echo $twig->render('connection.html', ['dVueErreur' => $dVueErreur]);
    }

    protected function inscription()
    {
        global $twig;
        $dVueErreur = []; // Tableau pour stocker les erreurs, le cas échéant
        $userModel = new \App\modele\UtilisateurModele();

        if (isset($_POST['firstname'],$_POST['name'], $_POST['email'], $_POST['password'])) {
            $nom = \App\config\Validation::nettoyerString($_POST['name']);
            $prenom = \App\config\Validation::nettoyerString($_POST['firstname']);
            $email = \App\config\Validation::nettoyerString($_POST['email']);
            $motDePasse = \App\config\Validation::nettoyerString($_POST['password']);
            $hash = password_hash($motDePasse, PASSWORD_DEFAULT);

            // verification que l'email est valide et unique :
            if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
                $dVueErreur[] = "L'adresse email n'est pas valide ou est déjà utilisée.";
            } else {
                $utilisateur = $userModel->getUtilisateurByEmail($email);
                if ($utilisateur instanceof \App\metier\Alumni) {
                    $dVueErreur[] = "L'adresse email est déjà utilisée.";
                }
            }
            $nouvelUtilisateur = $userModel->inscription($prenom,$nom,$email, $hash);

            if ($nouvelUtilisateur instanceof \App\metier\Alumni) {
                header('Location: index.php?action=inscription_success');
                exit();
            } else {
                $dVueErreur[] = "L'inscription a échoué, veuillez réessayer.";
            }
        }
        echo $twig->render('inscription.html', ['dVueErreur' => $dVueErreur]);
    }

    protected function accueil()
    {
        global $twig;
        // Ajout d'un var_dump pour déboguer
        if (isset($_SESSION['utilisateur']) && $_SESSION['utilisateur'] instanceof \App\metier\Alumni) {
            $prenom = $_SESSION['utilisateur']->getPrenom();
            $nom = $_SESSION['utilisateur']->getNom();
        }
        else{
            $prenom =  null;
            $nom = null;
        }
        //aller sur la page d'accueil avec le nom et prenom de l'utilisateur
        echo $twig->render('accueil.html', ['prenom' => $prenom, 'nom' => $nom]);
    }

    protected function inscription_success()
    {
        global $twig;
        echo $twig->render('inscription_success.html');
    }
    protected function consulterProfilLimite()
    {
        //TODO
    }

    /**
     * @return void
     * @brief Affiche la liste des offres d'emploi
     */

    protected function consultOffers()
    {

        global $twig;


        $niveauEtudes=null;
        $typeContrat=null;
        $exp = null;

        // number of offers by pages
        $nbOffers = 5 ;

        /** validation of entry's :  */
        if(isset($_GET["niveauEtudes"]) && Validation::validateTypeStudies($_GET["niveauEtudes"])) {
            $niveauEtudes = $_GET["niveauEtudes"];
        }

        if(isset($_GET["typeContrat"]) && Validation::validateTypeContract($_GET["typeContrat"])) {
            $typeContrat = $_GET["typeContrat"];
        }

        if(isset($_GET["experience"]) && Validation::validateExperience($_GET["experience"])) {
            $exp = $_GET["experience"];
        }

        if ($niveauEtudes == null && $typeContrat == null && $exp == null) {
            $totalOffers = $this->userModel->getNbOffers();
        } else {
            $params = array(
                'typeContrat' => $typeContrat,
                'exp' => $exp,
                'niveauEtudes' => $niveauEtudes,
            );
            $offers = $this->userModel->getOffersWithFilters($params);
            $totalOffers = count($offers);
        }


        // get the current pages to display
        $numberPages = ceil($totalOffers / 5);

        if($numberPages == 0 )
        {

            echo $twig->render("OffersList.html",['dVueErreur' => ['Aucune Offre Trouvée']]);
            return;
        }


        // validate the page number
        if (isset($_GET["page"]) && intval($_GET["page"]) != null) {
            $page = intval($_GET["page"]);
            if ($page > $numberPages || $page < 1) {
                $dVueErreur[] = "Page introuvable";
                echo $twig->render("erreur.html", ['dVueErreur' => $dVueErreur]);
                return;
            }
        } else {
            $page = 1;
        }


        $start = intval(($page - 1) * 5);

        if ($niveauEtudes == null && $typeContrat == null && $exp == null) {
            $offers = $this->userModel->getOfferLimit($start, $nbOffers);
        } else {
            $params['start'] = $start;
            $params['nbOffers'] = 5;
            $offers = $this->userModel->getOffersWithFilters($params);
        }



       /* echo "filtre :".$niveauEtudes."<br>";
        echo "filtre :".$typeContrat."<br>";
        echo "filtre :".$exp."<br>";*/

        echo $twig->render('OffersList.html', [
            'offres' => $offers,
            'numberPages' => $numberPages,
            'currentPage' => $page,
            'typeContrat' => (($typeContrat != null ? $typeContrat : "")),
            'experience' => (($exp != null ? $exp : "")),
            'niveauEtudes' => (($niveauEtudes != null ? $niveauEtudes : "")),
            'valContrat' => (($typeContrat != null ? "&typeContrat=".$typeContrat : "")),
            'valExp' => (($exp != null ? "&experience=".$exp : "")),
            'valEtudes' => (($niveauEtudes != null ? "&niveauEtudes=".$niveauEtudes : ""))
        ]);
    }




    // display the form to create an offer
    protected function createOfferForm()
    {
        global $twig;
        echo $twig->render("CreerOffre.html", []);
    }


    // create offer
    protected function createOffer()
    {
        global $twig;
        $taberror = [];

        $requiredFields = ['name', 'entreprise', 'description','typeContrat', 'descriptPoste', 'profilRecherche', 'choixExp', 'education', 'mail', 'num'];


        // validate required fields
        $error = false;
        foreach ($requiredFields as $field) {
            if (empty($_POST[$field])) {
                $error = true;
                $taberror[] = "Le champ {$field} est requis !";
            }
        }
        if($error)
        {
            echo $twig->render("CreerOffre.html", ['tabError' => $taberror ]);
            return;
        }


        // validate entrys
        if(!Validation::verifierEmail($_POST["mail"]))
        {
            //echo $twig->render("CreerOffre.html", ['errMsg' => "Email non valide !" ]);
            $taberror[] = "Email non valide !";
        }

        if(!Validation::validateNumber($_POST["num"]))
        {
            $taberror[] = "Numero non valide !";
            //echo $twig->render("CreerOffre.html", ['errMsg' => "Numero non valide !" ]);
        }


        if (!Validation::validateImage("image"))
        {
            if(isset($_FILES['image']['name'])) {$taberror[] = "Image non valide !";}
            else {$taberror[] = "Inserez une image !";}
        }
        if(!Validation::validateImage("logo")) {
            if(isset($_FILES['logo']['name'])) {$taberror[] = "Logo non valide !";}
            else {$taberror[] = "Inserez un logo !";}
            return;
        }


        // if errors display the form with errors
        if(count($taberror) > 0)
        {
            echo $twig->render("CreerOffre.html", ['tabError' => $taberror ]);

        }
        else{
            $saveImg1 = ImageSaver::SaveImage("image");
            $saveImg2 = ImageSaver::SaveImage("logo");
            if($saveImg1[0] && $saveImg2[0]) {

                $offre = $this->userModel->publishOffer($saveImg1[1], $saveImg2[1]);

                echo $twig->render("OffreDetailTest.html", ['offre' => $offre]);
            }
            else
            {
                $taberror[] = "Erreur lors de l'upload des images";
                echo $twig->render("CreerOffre.html", ['tabError' => $taberror ]);
            }
        }
    }

    // display offers
    public function displayOffer()
    {
        global $twig;

        if (isset($_GET["id"]) && intval($_GET["id"]) != null)
        {
            $offre = $this->userModel->getOfferFromId(intval($_GET["id"]));
            if($offre != NULL)
            {
                echo $twig->render("OffreDetailTest.html",['offre' => $offre]);
                return;
            }
        }
        $dVueErreur[] = "Erreur, Offre introuvable";
        echo $twig->render("erreur.html", ['dVueErreur' => $dVueErreur]);
    }

}